/**********************************************************************/
/*   Following  macro  used  to  allow  compilation  with ANSI C and  */
/*   non-ANSI C compilers automatically.			      */
/**********************************************************************/
# if !defined(PROTO)
# 	if	defined(__STDC__)
#		define	PROTO(x)	x
#	else
#		define	PROTO(x)	()
#	endif
# endif

typedef struct ref_t {
	char	*r_ptr;	/* Pointer to the object.			*/
	int	r_ref;	/* Number of times object being referenced.	*/
	OPCODE	r_type;	/* Base type of object.				*/
	int	r_used;	/* Number of bytes used by object.		*/
	int	r_size;	/* Bytes allocated for object.			*/
	} ref_t;

ref_t	*r_inc PROTO((ref_t *));
ref_t	*r_cat PROTO((ref_t *, char *));
ref_t	*r_append PROTO((ref_t *, char *, int, int));
ref_t	*r_inc PROTO((ref_t *));
ref_t	*r_init PROTO((OPCODE, char *, int));
void	r_dec PROTO((ref_t *));

