# include	"llist.h"

extern char *ll_freelist;
Head_p
ll_init()

{	register Head_p hp;

	if ((hp = (Head_p) vm_alloc(sizeof (Head), (void **) &ll_freelist)) == NULL)
		return NULL;
	LIST_INCR(hd_alloced);
	LIST_INCR(ll_init);
	hp->ll_magic = LL_HEAD;
	hp->ll_next = hp->ll_prev = NULL;
	hp->ll_elem = NULL;
	return hp;
}
