# include	"llist.h"

List_p	
ll_lookup(hp, str, func)
Head_p	hp;
char	*str;
int	(*func)();
{	register List_p	lp;

	LIST_INCR(ll_lookup);
	for (lp = hp->ll_next; lp; lp = lp->ll_next)
		if ((*func)(str, lp->ll_elem) == 0)
			return lp;
	return NULL;
}
