# include	"llist.h"

char *ll_freelist = NULL;
struct list_stats __list_stats;

List_p
ll_alloc()

{	register List_p lp;

	if ((lp = (List_p) vm_alloc(sizeof (List), &ll_freelist)) == NULL)
		return NULL;

	LIST_INCR(lp_alloced);
	LIST_INCR(ll_alloc);
	lp->ll_magic = LL_LIST;
	lp->ll_next = lp->ll_prev = NULL;
	return lp;
}
