#if	defined(SUN) || defined(SELECT)
# include	<time.h>
# if defined LINUX 
#	 include <linux/time.h>
# else
#   if defined __sparc__
#     include <sys/time.h>
#   else
#     include <sys/select.h>
#   endif
# endif
#endif

# if	defined(HAVE_TERMIO)
# 	include	<termio.h>
# endif

# if	defined(HAVE_SGTTY)
#	include	<sgtty.h>
# endif

# if	defined(SYSV) && defined(HAVE_PTY)
#	if CR_AIX_3
#		include	<sys/pty.h>
#	else
#		include <sys/stream.h>
#		include <sys/ptem.h>
#	endif
# endif

# if	!defined(SYSV) && !CR_SUN_OS41 && !CR_SUN_OS40 && !CR_LINUX
#	if defined(TCGETA)
#		undef	TCGETA
#		undef	TCSETA
#	endif
#	define	TCGETA		TIOCGETP
#	define	TCSETA		TIOCSETN
# endif

