/**************************************************************
 *
 *	CRISP - Custom Reduced Instruction Set Programmers Editor
 *
 *	(C) Paul Fox, 1989
 *
 *    Please See COPYRIGHT notice.
 *
 **************************************************************/
#define NSUBEXP  10
typedef struct REGEXP {
	char	*startp[NSUBEXP];
	char	*endp[NSUBEXP];
	int	length;		/* Length of matched string.	*/
	char	*start;
	char	*end;
	int	exp_level;	/* Highest level of {} parsed.	*/
	char	*setpos;
	char	*program;
	} REGEXP;
extern struct regopts {
	int	case_sense;	/* If TRUE do case sensitive search. 	*/
	int	regexp_chars;	/* If FALSE no regexp characters used.	*/
	int	unix_re;	/* If TRUE - Unix style regexps; else */
				/* CRISP syntax.		      */
	int	fwd_search;	/* If TRUE do a forward search; otherwise go */
				/* backwards.				     */
	int	offset;		/* Offset from start of line to start */
				/* search.			      */
	} regopts;
	
extern REGEXP *regcomp();

