# define	PARSE_ERR_ELLIPSIS_ERROR	-3
# define	PARSE_ERR_FLOAT_SYNTAX_ERROR	-2
# define	PARSE_ERR_EOF_IN_FLOAT		-1
# define	PARSE_INTEGER	0
# define	PARSE_FLOAT	1
# define	PARSE_ELLIPSIS	2
# define	PARSE_DOT	3

int	parse_number();
int	parse_str_number();
