#
# Awk script to take a list of primitives and sections and create the
# sections tables in help/obj for the on-line help.
	{
	for (i = 2; i <= NF; i++) {
		printf("echo '%s' >> obj/%s.sec\n", $1, $i);
		}
	}
