/**********************************************************************/
/*   Code to help me debug X stuff.				      */
/**********************************************************************/
# include	<X11/Xlib.h>
# include	<X11/Xutil.h>
# include	<X11/Xos.h>
# include	<X11/Xatom.h>

static char *event_names[] = {
	"Event 0",
	"Event 1",
	"KeyPress",
	"KeyRelease",
	"ButtonPress",
	"ButtonRelease",
	"MotionNotify",
	"EnterNotify",
	"LeaveNotify",
	"FocusIn",
	"FocusOut",
	"KeymapNotify",
	"Expose",
	"GraphicsExpose",
	"NoExpose",
	"VisibilityNotify",
	"CreateNotify",
	"DestroyNotify",
	"UnmapNotify",
	"MapNotify",
	"MapRequest",
	"ReparentNotify",
	"ConfigureNotify",
	"ConfigureRequest",
	"GravityNotify",
	"ResizeRequest",
	"CirculateNotify",
	"CirculateRequest",
	"PropertyNotify",
	"SelectionClear",
	"SelectionRequest",
	"SelectionNotify",
	"ColormapNotify",
	"ClientMessage",
	"MappingNotify"
	};
void
x_trace_event(event)
XEvent *event;
{
	printf("EVENT %d: %s\r\n", event->type, event_names[event->type]);
}
