/* #include <X11/copyright.h> */

/* $XConsortium: Template.h,v 1.4 89/07/21 01:41:49 kit Exp $ */
/* Copyright	Massachusetts Institute of Technology	1987, 1988 */

#ifndef _Crwin_h
#define _Crwin_h

# if	defined(X11r3)
#	define	XtPointer	void *
# endif
/****************************************************************
 *
 * Crwin widget
 *
 ****************************************************************/

/* Resources:

 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 background	     Background		Pixel		XtDefaultBackground
 border		     BorderColor	Pixel		XtDefaultForeground
 borderWidth	     BorderWidth	Dimension	1
 destroyCallback     Callback		Pointer		NULL
 height		     Height		Dimension	0
 mappedWhenManaged   MappedWhenManaged	Boolean		True
 sensitive	     Sensitive		Boolean		True
 width		     Width		Dimension	0
 x		     Position		Position	0
 y		     Position		Position	0

crwinWidget:
 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 font		     Font		String		"fixed"
insertCursorColor    InsertCursorColor	Pixel		"red"
overtypeCursorColor  OvertypeCursorColor Pixel		"yellow"
 
*/

/* define any special resource names here that are not in <X11/StringDefs.h> */

#define XtNcrwinResource "crwinResource"
#define	XtNscreenBackground	"screenBackground"
#define	XtNtextColor	"textColor"
#define	XtNselectedWindow	"selectedWindow"
#define	XtNnormalMessages	"normalMessages"
#define	XtNerrorMessages	"errorMessages"
#define	XtNhilightForeground	"hilightForeground"
#define	XtNhilightBackground	"hilightBackground"
#define	XtNinsertCursorColor	"insertCursorColor"
#define	XtNovertypeCursorColor	"overtypeCursorColor"

#define	XtCTextColor	"TextColor"
#define	XtCWindowBackground	"WindowBackground"
#define	XtCSelectedWindow	"SelectedWindow"
#define	XtCNormalMessages	"NormalMessages"
#define	XtCErrorMessages	"ErrorMessages"
#define	XtCHilightForeground	"HilightForeground"
#define	XtCHilightBackground	"HilightBackground"
#define XtCCrwinResource "CrwinResource"
#define	XtCInsertCursorColor	"InsertCursorColor"
#define	XtCOvertypeCursorColor	"OvertypeCursorColor"

/* declare specific CrwinWidget class and instance datatypes */

typedef struct _CrwinClassRec*	CrwinWidgetClass;
typedef struct _CrwinRec*		CrwinWidget;

/* declare the class constant */

extern WidgetClass crwinWidgetClass;

/**********************************************************************/
/*   Following  macro  used  to  allow  compilation  with ANSI C and  */
/*   non-ANSI C compilers automatically.			      */
/**********************************************************************/
# if !defined(PROTO)
# 	if	defined(__STDC__)
#		define	PROTO(x)	x
#	else
#		define	PROTO(x)	()
#	endif
# endif

void	crwin_draw_string PROTO((Widget, int, int, char *, int, Pixel, Pixel));
void	crwin_get_window_size PROTO((Widget, int *, int *));
void	crwin_show_cursor PROTO((Widget, int, int, int, char *, int));
int	crwin_getkey PROTO((Widget, int));
extern Pixel x11_colors[];
#endif /* _Crwin_h */
