\" .Fc title version date	Cover sheet
\" .Fo section_name		Used for Name/Synopsis/Description
.tr ~
.de Fc				\" Define Standard Cover sheet.
.ll 6i
.pl 10.5i
.po 1i
.ev2
.ll 6i
.ev
.nr Ej 1
.SA 1
.nr % -1
.DS















.DE
.ce
.B
\s+4\\$1\s-4
.DS CB


Author:		P. D. Fox
Version:        \\$2
Date:           \\$3










.DE
.ds HF 3 3 3 3 3
.ce
.ul
.B "(C) Paul Fox, 1988"
.sp 1
.ll -7m
.VL 7
.LI \ 
This document forms part of the documentation set of the
CRISP editor. The editor and this code is placed in the public
domain on the understanding that it may not be sold without
prior consent to the author. Please refer to the CRISP Copyright
notice for further details of the distribution conditions.
.LE
.ll +7m
.PH "'\\$1''Page %'"
.PF "'CRISP Documentation'Version \\$2 - \\$3''"
.bp
..
.de Fo				\" Negative indent Name/Synopsis etc.
.sp
.in -.5i
.B \\$1
.br
.in +.5i
.sp
..
.de EL
.LE
..
