/**********************************************************************/
/*   Standard Unix library routines not available under the Vax.      */
/**********************************************************************/

_assert(str, file, line)
char *str;
char *file;
{
	printf("Assertion failed '%s', in %s at line %d\n",
		str, file, line);
	exit(1);
}
getcwd(buf, size)
char *buf;
{	char	*getenv();
	char	*cp = getenv("PATH");
	
	strcpy(buf, cp);
}
qsort()
{
}
_cleanup()
{
}
/* Force CRISP to think link failed so we do a normal copy. */
link(f1, f2)
char *f1, *f2;
{
	return -1;
}
/* Version of unlink for VMS */
unlink(filename)
char *filename;
{
	return delete(filename);
}
