# if	!defined(__SYSCALLS)
# define	__SYSCALLS	0	/* Make this non-zero to trace  */
					/* system calls.		*/ 
# endif
# if	!defined(__TLI)
# define	__TLI		0  
# endif
# define	__TLI_ALLOC	0	/* Log t=alloc() and t_free() calls */ 

# if __TLI 
 
# define	 t_accept(fd, fd1, x)	_tli_print(0, "t_accept", fd, fd1, x, 0, 3)  /**/
# define	 t_alloc(fd, n1, n2) 	_tli_print(1, "t_alloc", fd, n1, n2, 0, 3)  /**/
# define	 t_bind(fd, req, ret) 	_tli_print(2, "t_bind", fd, req, ret, 0, 3) /**/
# define	 t_close(fd)		_tli_print(3, "t_close", fd, 0, 0, 0, 1) /**/
# define	 t_connect(fd, x, y)	_tli_print(4, "t_connect", fd, x, y, 0, 3) /**/
# define	 t_free(x, y) 		_tli_print(5, "t_free", x, y, 0, 0, 2) /**/
# define	 t_listen(fd, x)	_tli_print(6, "t_listen", fd, x, 0, 0, 2) /**/
# define	 t_look(fd) 		_tli_print(7, "t_look", fd, 0, 0, 0, 1) /**/
# define	 t_open(path, mode, x) 	_tli_print(8, "t_open", path, mode, x, 0, 3) /**/
# define	 t_rcv(fd, buf, len, flags) _tli_print(9, "t_rcv", fd, buf, len, flags, 4) /**/
# define	 t_rcvconnect(fd, call) _tli_print(10, "t_rcvconnect", fd, call, 0, 0, 2) /**/
# define	 t_rcvdis(fd, x)	_tli_print(11, "t_rcvdis", fd, x, 0, 0, 2) /**/
# define	 t_rcvresetcfm(fd)	_tli_print(12, "t_rcvresetcfm", fd, 0, 0, 0, 1) /**/
# define	 t_reset(fd, diag)	_tli_print(13, "t_reset", fd, diag, 0, 0, 2) /**/
# define	 t_snd(fd, buf, len, x)	_tli_print(14, "t_snd", fd, buf, len, x, 4) /**/
# define	 t_snddis(fd, x)	_tli_print(15, "t_snddis", fd, x, 0, 0, 2) /**/
# define	 t_unbind(fd) 		_tli_print(16, "t_unbind", fd, 0, 0, 0, 1) /**/
# endif

# if	__SYSCALLS
#	define	CLOSE(fd) 	_CLOSE(__FILE__, __LINE__, fd)
#	define	EXIT(n)		_EXIT(__FILE__, __LINE__, n)
#	define	FCNTL(fd, a, b)	_FCNTL(__FILE__, __LINE__, fd, a, b)
#	define	LOCKF(fd, a, b) _LOCKF(__FILE__, __LINE__, fd, a, b)
#	define	LSEEK(fd, a, b) _LSEEK(__FILE__, __LINE__, fd, a, b)
#	define	MKNOD(name, a, b) _MKNOD(__FILE__, __LINE__, name, a, b)
#	define	OPEN(name, a, b) _OPEN(__FILE__, __LINE__, name, a, b)
#	define	READ(fd, buf, len) _READ(__FILE__, __LINE__, fd, buf, len)
#	define	WRITE(fd, buf, len) _WRITE(__FILE__, __LINE__, fd, buf, len)
# else
#	define	CLOSE 	close   
#	define	EXIT	exit
#	define	FCNTL	fcntl 
#	define	LOCKF	lockf
#	define	LSEEK	lseek 
#	define	MKNOD	mknod
#	define	OPEN	open
#	define	READ 	read
#	define	WRITE	write
# endif
