/*
/*   Macros to help in drawing diagrams.
/**/
void
x()
{
	box(8, 4);
}
void
box(int length, int height)
{
	int	i, line, col;
	
	insert_mode();
		
	inq_position(line, col);
	self_insert('+');
	
	for (i = 0; i < length; i++)
		self_insert('-');
	self_insert('+');
	
	for (i = 0; i < height; i++) {
		down();
		left();
	        self_insert('|');
		}

	move_abs(line+1, col);
	message("line=%d col=%d", line, col);
	
	for (i = 1; i < height; i++) {
        	self_insert('|');
		down();
		left();
		}
	
	move_abs(line + height, col);
	self_insert('+');
	for (i = 0; i < length; i++)
		self_insert('-');
	self_insert('+');
	insert_mode();
}
