/**************************************************************
 *
 *	CRISP - Custom Reduced Instruction Set Programmers Editor
 *
 *	(C) Paul Fox, 1989
 *
 *    Please See COPYLEFT notice.
 *
 **************************************************************/
# define	SECONDS		*1000
# define	SECOND		*1000
# define	MILLISECONDS

# define	WAIT_STATE_0	0x01  /* Zero delay -- see if key pressed. */
# define	WAIT_STATE_1	0x02  /* Wait for first key pressed.	*/
# define	WAIT_STATE_2	0x04  /* Wait for subsequent key of a function key */
# define	WAIT_IDLE	0x08  /* Idle timer active.		*/
# define	WAIT_PTY	0x10  /* Pty active - need to sample at 1sec interval. */
# define	WAIT_MINUTE	0x20  /* Minute passed.			*/
# define	WAIT_WAIT	0x40  /* (wait) or (sleep) macro.	*/
# define	WAIT_TIMEOUT	0x80  /* Timeout past to getkey() expired. */

# define	WAIT_KEYS	(WAIT_STATE_0 | WAIT_STATE_1 | WAIT_STATE_2)

extern int wait_state;
