char *
memcpy(dst, src, cnt)
register char	*dst;
register char	*src;
register int cnt;
{	register char *cp = dst;
	register int	lcnt = cnt >> 2;

	cnt &= 3;
	while (lcnt-- > 0)
		*((long *) dst)++ = *((long *)src)++;
	while (cnt-- > 0)
		*dst++ = *src++;
	return cp;
}

