# include	<string.h>

char *
instr(str1, str2)
char	*str1, *str2;

{	int len2 = strlen(str2);

	while (*str1) {
		if (strncmp(str1, str2, len2) == 0)
			return str1;
		str1++;
		}
	return (char *) 0;
}
