/********************************************************************
 *                                                                  *
 *      CRISP - Custom Reduced Instruction Set Programmers Editor   *
 *                                                                  *
 *    Please See COPYRIGHT notice.                                  *
 *                                                                  *
 *  This file contains the keyboard labels for a SCO keyboard when  *
 *  the strings and keys files are used.			    *
 *                                                                  *
 ********************************************************************/

# include	<alt.h>

list kbd_labels = {
AF(1),			"<Ctrl-Shift-F1>",
AF(2),			"<Ctrl-Shift-F2>",
AF(3),			"<Ctrl-Shift-F3>",
AF(4),			"<Ctrl-Shift-F4>",
AF(5),			"<Ctrl-Shift-F5>",
AF(6),			"<Ctrl-Shift-F6>",
AF(7),			"<Ctrl-Shift-F7>",
AF(8),			"<Ctrl-Shift-F8>",
AF(9),			"<Ctrl-Shift-F9>",
AF(10),			"<Ctrl-Shift-F10>",
AF(11),			"<Ctrl-Shift-F11>",
AF(12),			"<Ctrl-Shift-F12>",
__SHIFT_KEYPAD(0),	"<Cntrl-Shift-Ins>",
__SHIFT_KEYPAD(1),	"<Cntrl-Shift-End>",
__SHIFT_KEYPAD(2),	"<Cntrl-Shift-Down>",
__SHIFT_KEYPAD(3),	"<Cntrl-Shift-PageDown>",
__SHIFT_KEYPAD(4),	"<Cntrl-Shift-Left>",
__SHIFT_KEYPAD(6),	"<Cntrl-Shift-Right>",
__SHIFT_KEYPAD(7),	"<Cntrl-Shift-Home>",
__SHIFT_KEYPAD(8),	"<Cntrl-Shift-Up>",
__SHIFT_KEYPAD(9),	"<Cntrl-Shift-PageUp>",
};
