rem Quick and dirty compile for OS/2
cl -G2 -O -Gs -AL -c -I../foxlib *.c
del ..\lib\llist.lib
lib ..\lib\llist.lib + alloc append clear delete elem first free hook init ;
lib ..\lib\llist.lib + insert look magic next prev push remove unhook;
