# if !defined(VERSION)
#	define	VERSION	99
# endif

# if CR_XENIX && VERSION < 23
# 	include <ndir.h>
# else
# if	defined(VMS)
# 	include	"vmsdir.h"
# else
#	if !defined(NO_SYS_DIR_H)
# 		include	<sys/dir.h>
#	endif
# endif
# endif

# if	defined(DIRSIZ_DEFINED)
#	if	defined(DIRSIZ)
#		undef	DIRSIZ
#	endif
#	define	DIRSIZ	MAXNAMLEN
#	if !CR_AIX_3
#		define	dirent direct
#	endif
# else
# 	include	<dirent.h>
# endif

# if CR_ACORN
# 	undef DIRSIZ
# 	define	DIRSIZ	MAXNAMLEN
# endif
