# This file should be sourced by your .login if you are using the
# C-shell. It is a good idea to source this from your .cshrc as well
# if you run a windowing system otherwise your terminal emulator may not
# see these.

# This file is set up for running under X-windows using 'xterm -name xcrisp'
# Please consult installation notes if you need to change anything.

setenv CRISP_ROOT ~/crisp

setenv BBACKUP ~/backup
setenv BHELP $CRISP_ROOT/help
setenv BKBD xterm
setenv BPATH $CRISP_ROOT/macros
setenv BTERM xcrisp
setenv BVERSION 6
setenv BTERMCAP $CRISP_ROOT/utils/termcap

