# include	"chkalloc.h"
char	*
strdup(str)
char	*str;
{
	extern	char	*memcpy();
	register char	*cp = str;
	register int	l = 0;

	while (*cp++)
		l++;
	l++;
	cp = chk_alloc(l);

	if (cp)
		return memcpy(cp, str, l);
	return 0;
}

