/********************************************************************
 *                                                                  *
 *      CRISP - Custom Reduced Instruction Set Programmers Editor   *
 *                                                                  *
 *      (C) Paul Fox, 1989, 1990                                    *
 *                                                                  *
 *    Please See COPYRIGHT notice.                                  *
 *                                                                  *
 *  CRISP keyboard definitions for use in the tty-*.m macros.	    *
 *                                                                  *
 ********************************************************************/

# define	TRUE			1
# define	FALSE			0

# include	"../../../alt.h"

# define	CTRL_A_Z		  1
# define	F1_F12			F(1)
# define	SHIFT_F1_F12		SF(1)
# define	CTRL_F1_F12		CF(1)
# define	ALT_F1_F12		AF(1)
# define	ALT_A_Z			ALT_A
# define	KEYPAD_0_9		KEYPAD_0
# define	DEL			KEY_DEL
# define	COPY			KEY_COPY
# define	CUT			KEY_CUT
# define	CTRL_KEYPAD_0_9		CTRL_KEYPAD_0
# define	ALT_0_9			ALT_0

