# include	<string.h>

char	*
dname(file)
register	char	*file;
{
	static	char	buf[64];
	register char *cp;
	strcpy(buf, file);

	cp = buf+strlen(buf)-1;
	for (; cp > buf; cp--)
		if (*cp == '/') {
			*cp = 0;
			break;
			}
	return buf;
}
