/**********************************************************************/
/*   Header file for debug_support() macro			      */
/**********************************************************************/

# define	DBG_STACK_TRACE	0
# define	DBG_NEST_LEVEL	1	/* Level of nesting of execution stack */
# define	DBG_INQ_VARS	2	/* Return list of variable values.	*/
# define	DBG_INQ_VAR_INFO 3	/* Return info about an actual variable. */
