#include "crisp.h"

void
check_dos_file()
{
	string curline;
	string filename;


	curline = trim(ltrim(read()));
	if (index(curline, "\r")) {
		inq_names(filename);
		delete_buffer(inq_buffer());
		edit_file(EDIT_CR, filename);
	}
}

void
dosedit()
{
	register_macro(REG_NEW, "check_dos_file");
}

