# include	"llist.h"

void
ll_delete(lp)
List_p	lp;
{	extern char *ll_freelist;

	LIST_INCR(ll_delete);
	LIST_DECR(lp_alloced);
	LL_MAGIC(lp);

	lp->ll_prev->ll_next = lp->ll_next;
	if (lp->ll_next)
		lp->ll_next->ll_prev = lp->ll_prev;
	vm_free((void *) lp, (void **) &ll_freelist);
}
