# This file should be read by your .profile or .bashrc file if you are
# using a Bourne shell derivative.

# This file is set up for running under X-windows using 'xterm -name xcrisp'
# Please consult installation notes if you need to change anything.
CRISP_ROOT=~/crisp

BBACKUP=~/backup
BHELP=$CRISP_ROOT/help
BKBD=xterm
BPATH=$CRISP_ROOT/macros
BTERM=xcrisp
BVERSION=6
BTERMCAP=$CRISP_ROOT/utils/termcap

export CRISP_ROOT BBACKUP BHELP BKBD BPATH BTERM BVERSION BTERMCAP
