# include	"llist.h"

char	*
ll_push(hp, atom)
Head_p	hp;
char	*atom;
{
	LIST_INCR(ll_push);
	ll_insert(hp, atom, 0);
	return atom;
}
List_p
ll_pop(hp)
Head_p	hp;
{	List_p	lp = ll_first(hp);

	LIST_INCR(ll_pop);
	if (lp)
		ll_delete(lp);

	return ll_first(hp);
	
}
