/**********************************************************************/
/*   Following  prototypes  defined  because many Unix systems don't  */
/*   declare  what  they  should  for  ANSI  C compatability. Needed  */
/*   only to shut up GNU C.					      */
/**********************************************************************/

# if defined(__STDC__) || defined(__GNUC__)

# if !defined(size_t) && !defined(_T_SIZE) && !defined(_SIZE_T) && !defined(_SIZE_T_DEFINED)
#	define	size_t	int
# endif

/**********************************************************************/
/*   Following  macro  used  to  allow  compilation  with ANSI C and  */
/*   non-ANSI C compilers automatically.			      */
/**********************************************************************/
# if !defined(PROTO)
# 	if	defined(__STDC__)
#		define	PROTO(x)	x
#	else
#		define	PROTO(x)	()
#	endif
# endif

#if !CR_LINUX
int	mkdir PROTO((char *, int));
#endif
char	*strtok();
char	*strdup PROTO((char const *));
char *	strrchr PROTO((const char *, int));
FILE	*popen PROTO((const char *, const char *));
int	pclose PROTO((FILE *));
int	rename PROTO((const char *, const char *));
int	execv PROTO((char *, char **));
char	*strpbrk();
int	system();
int	fputs();
int	fgetc PROTO((FILE *));
void	rewind PROTO((FILE *));
int	getopt PROTO((int, char **, char *));
int	dup PROTO((int));
long	ftell PROTO((FILE *));
char	*mktemp PROTO((char *));
char	*strchr PROTO((char const *, int));
int	_flsbuf();
int	access PROTO((char *, int));
int	chdir PROTO((char *));
int	chown PROTO((char *, int, int));
int	close PROTO((int));
int	execl();
int	execlp();
int	fork PROTO((void));
int	fprintf PROTO((FILE *, char const *, ...));
int	fputc PROTO((int, FILE *));
size_t	fread PROTO((void *, size_t, size_t, FILE *));
int	fseek PROTO((FILE *, long, int));
size_t	fwrite PROTO((void const *, size_t, size_t, FILE *));
int	getgid PROTO((void));
int	getpid PROTO((void));
int	getuid PROTO((void));
int	ioctl();
int	link PROTO((char *, char *));
int	lseek PROTO((int, long, int));
int	pipe PROTO((int *));
int     printf PROTO((char const *, ...));
int	read PROTO((int, void *, int));
int	setpgrp();
int	siginterrupt PROTO((int, int));
int	sigsetmask PROTO((int));
size_t	strcspn PROTO((char const *, char const *));
int	unlink PROTO((const char *));
int	wait();
int	write PROTO((int, void *, int));
void	_exit PROTO((int));
void	endpwent PROTO((void));
int	fclose PROTO((FILE *));
int	fflush PROTO((FILE *));
void   	perror PROTO((char const *));

# endif
