/**********************************************************************/
/*   This  file  is  included  if we're running under X11 release 3.  */
/*   We try and hide the differences between release 4 and 3.	      */
/**********************************************************************/
# if (defined(X11r3) || defined(i386) || defined(SYSV)) && !defined __linux__
#	define	XtAppPending(app_con)	XtPending()
#	define	XtAppInitialize(app_con, name, arg1, arg2, argc, argv, fallback, arg3, arg4) \
		XtInitialize("crisp", name, arg1, arg2, argc, argv)
#	define	XtAppNextEvent(app_con, event)	XtNextEvent(event)
# if !defined(XtPointer)
#		define	XtPointer	char *
# endif
# endif
