char *
bsearch(name, tbl, size, sizeof_one, compare)
char *name;
char *tbl;
int (*compare)();
{	int	l = 0;
	int	u = size-1;
	int	res, m;
	char	*p;
	
	while (1) {
		if (l > u)
			return (char *) 0;
		m = (l + u) / 2;
		p = tbl + (sizeof_one * m);
		res = (*compare)(name, p);
		if (res > 0)
			l = m + 1;
		else if (res == 0)
			return p;
		else
			u = m - 1;
		}
	
}

